package function.builtin

import environment.RuntimeEnvironment
import function.ArgumentValidator.TooManyArgumentsException
import org.junit.Assert.assertFalse
import org.junit.Assert.assertTrue
import org.junit.Test
import testutil.SymbolAndFunctionCleaner
import testutil.TestUtilities.evaluateString
import java.util.HashSet

class EXITTest : SymbolAndFunctionCleaner() {
    private val environment: RuntimeEnvironment
    private var indicatorSet: MutableSet<String>? = null

    init {
        this.environment = RuntimeEnvironment
    }

    private fun assertTerminated() {
        assertTrue(indicatorSet!!.contains(TERMINATED))
    }

    private fun assertNotTerminated() {
        assertFalse(indicatorSet!!.contains(TERMINATED))
    }

    override fun additionalSetUp() {
        indicatorSet = HashSet()
        environment.reset()
        environment.terminationFunction = { indicatorSet!!.add(TERMINATED) }
    }

    override fun additionalTearDown() {
        environment.reset()
    }

    @Test
    fun exitWorks() {
        evaluateString("(exit)")
        assertTerminated()
    }

    @Test
    fun exitNotCalled_IndicatorSetIsClean() {
        assertNotTerminated()
    }

    @Test(expected = TooManyArgumentsException::class)
    fun exitWithTooManyArguments() {
        evaluateString("(exit 1)")
    }

    companion object {

        private val TERMINATED = "terminated"
    }
}
